@@ -30,7 +30,7 @@ void kde_tray_init(Display *dpy)
 	 * All windows that are listed there are considered to be "old" KDE icons,
 	 * i.e. icons that are to be ignored on the tray startup.
 	 * If the property does not exist, fall back to old mode */
-	if (tray_data.xa_kde_net_system_tray_windows != None &&
+	if (tray_data.xa_kde_net_system_tray_windows == None ||
 	    !x11_get_root_winlist_prop(dpy, tray_data.xa_kde_net_system_tray_windows, 
 				(unsigned char **) &old_kde_icons, &n_old_kde_icons)) 
 	{
@@ -42,7 +42,10 @@ void kde_tray_init(Display *dpy)
 
 	/* If theres no previous tray selection owner, try to embed all available
 	 * KDE icons and, therefore, leave the list of old KDE icons empty */
-	if (tray_data.old_selection_owner == None) return;
+	if (tray_data.old_selection_owner == None) {
+		n_old_kde_icons = 0;
+		return;
+	}
 
 	/* Next, we are going to remove some entries from old_kde_icons list */
 
@@ -131,7 +134,6 @@ int kde_tray_check_for_icon(Display *dpy, Window w)
 			False, XA_WINDOW, &actual_type, &actual_format, &nitems, &bytes_after,
 			&data);
 	XFree(data);
-
 	if (x11_ok() && actual_type == XA_WINDOW && nitems == 1)
 		return True;
 	else
