@@ -261,12 +261,14 @@ int find_invalid_icons(struct TrayIcon *ti)
 
 #ifndef NO_NATIVE_KDE
 /* Find newly available KDE icons and add them as necessary */
+/* TODO: move to kde_tray.c */
 void kde_icons_update()
 {
 	unsigned long list_len, i;
 	Window *kde_tray_icons;
 
-	if (!x11_get_root_winlist_prop(tray_data.dpy, tray_data.xa_kde_net_system_tray_windows, 
+	if (tray_data.kde_tray_old_mode || 
+		!x11_get_root_winlist_prop(tray_data.dpy, tray_data.xa_kde_net_system_tray_windows, 
 				(unsigned char **) &kde_tray_icons, &list_len)) 
 	{
 		return;
@@ -344,7 +346,7 @@ void perform_periodic_tasks()
 		if (!tray_data.is_reparented && 
 				(xwa.width != tray_data.xsh.width || xwa.height != tray_data.xsh.height)) 
 		{
-			DBG(8, ("KLUDGE: fixing window size (current: %dx%d, required: %dx%d\n",
+			DBG(8, ("KLUDGE: fixing window size (current: %dx%d, required: %dx%d)\n",
 						xwa.width, xwa.height,
 						tray_data.xsh.width, tray_data.xsh.height));
 			tray_update_window_size();
@@ -419,7 +421,7 @@ void property_notify(XPropertyEvent ev)
 					break;
 				}
 		}
-		DBG(4, ("tray is %sreparented\n", tray_data.is_reparented ? "" : "not "));
+		DBG(6, ("tray is %sreparented\n", tray_data.is_reparented ? "" : "not "));
 	}
 }
 
@@ -541,27 +543,20 @@ void configure_notify(XConfigureEvent ev)
 {
 	struct TrayIcon *ti;
 	struct Point sz;
-	int x = 0, y = 0;
+	XWindowAttributes xwa;
 
 	if (ev.window == tray_data.tray) {
+		/* Tray window was resized */
 		/* TODO: distinguish between synthetic and real configure notifies */
 		/* TODO: catch rejected configure requests */
-		/* Tray window was resized */
 		DBG(8, ("window geometry from event: %ux%u+%d+%d\n", ev.width, ev.height, ev.x, ev.y));
-	
-		/* Update tray window coordinates */
-		if (x11_get_window_abs_coords(tray_data.dpy, tray_data.tray, &x, &y)) {
-			DBG(8, ("absolute position: +%d+%d\n", x, y));
-			tray_data.xsh.x = x;
-			tray_data.xsh.y = y;
-		} else 
-			DBG(0, ("could not get tray`s absolute coords\n"));
 
-		tray_data.xsh.width = ev.width;
-		tray_data.xsh.height = ev.height;
-
-		DBG(6, ("current window geometry: %ux%u+%d+%d\n", 
-				tray_data.xsh.width, tray_data.xsh.height, tray_data.xsh.x, tray_data.xsh.y));
+		/* Sometimes, configure notifies come too late, so we fetch real geometry ourselves */
+		XGetWindowAttributes(tray_data.dpy, tray_data.tray, &xwa);
+		x11_get_window_abs_coords(tray_data.dpy, tray_data.tray, &tray_data.xsh.x, &tray_data.xsh.y);
+		DBG(8, ("real window geometry: %dx%d+%d+%d\n", xwa.width, xwa.height, tray_data.xsh.x, tray_data.xsh.y));
+		tray_data.xsh.width = xwa.width;
+		tray_data.xsh.height = xwa.height;
 
 		/* Update icons positions */
 		icon_list_forall(&grid2window);
@@ -589,6 +584,7 @@ void configure_notify(XConfigureEvent ev)
 		ti->l.wnd_sz = sz;
 		ti->is_resized = True;
 		/* Do the job */
+#if 0
 		if (layout_handle_icon_resize(ti)) {
 			embedder_refresh(ti);
 #ifdef DEBUG
@@ -597,6 +593,15 @@ void configure_notify(XConfigureEvent ev)
 			embedder_update_positions(False);
 			tray_update_window_size();
 		}
+#else
+		layout_handle_icon_resize(ti);
+		embedder_refresh(ti);
+#ifdef DEBUG
+		print_icon_data(ti);
+#endif
+		embedder_update_positions(False);
+		tray_update_window_size();
+#endif
 #ifdef DEBUG
 		dump_icon_list();
 #endif
