@@ -288,50 +288,50 @@ void usage(char *progname)
 			"argument, which can be true (1, yes) or false (0, no).\n"
 			"\n"
 			"Possible options are:\n"
-			"    -display <display>         use X display <display>\n"
+			"    -display <display>          use X display <display>\n"
 #ifdef DEBUG
-			"    --dbg-level <n>            set the level of debug output to <n>\n"
-			"                               (defalt: 0)\n"
+			"    --dbg-level <n>             set the level of debug output to <n>\n"
+			"                                (defalt: 0)\n"
 #endif
-			"    -bg, --background <color>  select background color (default: #777777)\n"
-			"    -c, --config <filename>    read configuration from <file>\n"
-			"                               (instead of default $HOME/.stalonetrayrc)\n"
-			"    -d, --decorations [<deco>] set what part of window decorations are\n"
-			"                               visible; deco can be: none (default),\n"
-			"                               title, border, all\n"
-			"    --fuzzy-edges [<level>]    set edges fuzziness, level is from\n"
-			"                               0 (disabled) to 3 (maximum); works with\n"
-			"                               tinting and/or pixmap background\n"
-			"    -geometry <geometry>       set initial tray`s geometry\n"
-			"    --grow-gravity <gravity>   tray`s grow gravity,\n"
-			"                               one of N, S, W, E, NW, NE, SW, SE\n"
-			"    --icon-gravity <gravity>   icon positioning gravity (NW, NE, SW, SE)\n"
-			"    -i, --icon-size <n>        set basic icon size to <n>\n"
-			"    --ignore-icon-resize       ignore icons attempts to resize their windows\n"
-			"    -h, --help                 show this message\n"
-			"    --max-width <n>            set tray`s width limit to <n>\n"
-			"                               (default: 0 = unlimited)\n"
-			"    --max-height <n>           set tray`s height limit to <n>\n"
-			"                               (default: 0 = unlimited)\n"
-			"    --no-shrink                do not shrink window back after icon removal\n"
-			"    -p, --parent-bg            use parent for background\n"
-			"    --pixmap-bg <pixmap>       use pixmap for tray`s window background\n"
-			"    --respect-icon-hints       try to respect icon size hints\n"
-			"    --skip-taskbar             hide tray`s window from the taskbar\n"
-			"    --sticky                   make tray`s window sticky across multiple\n"
-			"                               desktops/pages\n"
-			"    --tint-color <color>       tint tray background with color (not used\n"
-			"                               with plain color background)\n"
-			"    --tint-level <level>       set tinting level from 0 to 255\n"
-			"    -t, --transparent          enable root transparency\n"
-			"    -v, --vertical             use vertical layout of icons (horizontal\n"
-			"                               layout is used by default)\n"
-			"    --window-layer <layer>     set tray`s window EWMH layer, one of:\n"
-			"                               bottom, normal, top\n"
-			"    --window-type <type>       set tray`s window EWMH type, one of:\n"
-			"                               normal, dock, toolbar, utility\n"
-			"    -w, --withdrawn            start in withdrawn mode as WMaker dockapp\n"
-			"    --xsync                    operate on X server synchronously (SLOW)\n"
+			"    -bg, --background <color>   select background color (default: #777777)\n"
+			"    -c, --config <filename>     read configuration from <file>\n"
+			"                                (instead of default $HOME/.stalonetrayrc)\n"
+			"    -d, --decorations [<deco>]  set what part of window decorations are\n"
+			"                                visible; deco can be: none (default),\n"
+			"                                title, border, all\n"
+			"    -f, --fuzzy-edges [<level>] set edges fuzziness, level is from\n"
+			"                                0 (disabled) to 3 (maximum); works with\n"
+			"                                tinting and/or pixmap background\n"
+			"    -geometry <geometry>        set initial tray`s geometry\n"
+			"    --grow-gravity <gravity>    tray`s grow gravity,\n"
+			"                                one of N, S, W, E, NW, NE, SW, SE\n"
+			"    --icon-gravity <gravity>    icon positioning gravity (NW, NE, SW, SE)\n"
+			"    -i, --icon-size <n>         set basic icon size to <n>, default: 24\n"
+			"    --ignore-icon-resize        ignore icons attempts to resize their windows\n"
+			"    -h, --help                  show this message\n"
+			"    --max-width <n>             set tray`s width limit to <n>\n"
+			"                                (default: 0 = unlimited)\n"
+			"    --max-height <n>            set tray`s height limit to <n>\n"
+			"                                (default: 0 = unlimited)\n"
+			"    --no-shrink                 do not shrink window back after icon removal\n"
+			"    -p, --parent-bg             use parent for background\n"
+			"    --pixmap-bg <pixmap>        use pixmap for tray`s window background\n"
+			"    --respect-icon-hints        try to respect icon size hints\n"
+			"    --skip-taskbar              hide tray`s window from the taskbar\n"
+			"    --sticky                    make tray`s window sticky across multiple\n"
+			"                                desktops/pages\n"
+			"    --tint-color <color>        tint tray background with color (not used\n"
+			"                                with plain color background)\n"
+			"    --tint-level <level>        set tinting level from 0 to 255\n"
+			"    -t, --transparent           enable root transparency\n"
+			"    -v, --vertical              use vertical layout of icons (horizontal\n"
+			"                                layout is used by default)\n"
+			"    --window-layer <layer>      set tray`s window EWMH layer, one of:\n"
+			"                                bottom, normal, top\n"
+			"    --window-type <type>        set tray`s window EWMH type, one of:\n"
+			"                                normal, dock, toolbar, utility\n"
+			"    -w, --withdrawn             start in withdrawn mode as WMaker dockapp\n"
+			"    --xsync                     operate on X server synchronously (SLOW)\n"
 			"\n"
 			);
 }
@@ -673,6 +673,8 @@ void interpret_settings()
 #endif
 
 	/* Parse geometry-related settings */
+	/* Since WMs do not handle gravity reliable,
+	 * calculate window position by hand */
 	geom_flags = XParseGeometry(settings.geometry_str, 
 					&tray_data.xsh.x, &tray_data.xsh.y,
 					(unsigned int *) &tray_data.xsh.width, 
