@@ -63,6 +63,7 @@ int ewmh_check_support(Display *dpy)
 		atom_name = XGetAtomName(dpy, ewmh_list[i]);
 		DBG(8, ("_NET_WM_SUPPORTED[%d] = 0x%x (%s)\n", i, ewmh_list[i], atom_name));
 		XFree(atom_name);
+		x11_ok();
 	}
 #endif
 	
@@ -93,7 +94,7 @@ int ewmh_add_window_state(Display *dpy, Window wnd, char *state)
 	atom = XInternAtom(dpy, state, True);
 	if (atom == None || prop == None || !ewmh_atom_supported(atom)) return FAILURE;
 
-	DBG(8, ("adding window state %s (0x%x)\b", state, atom));
+	DBG(8, ("adding window state %s (0x%x)\n", state, atom));
 
 	rc = XGetWindowAttributes(dpy, wnd, &xwa);
 	if (!x11_ok() || !rc ) return FAILURE;
@@ -189,6 +190,7 @@ int mwm_set_hints(Display *dpy, Window wnd, unsigned long decorations, unsigned
 		prop = &new_prop;
 	} else {
 		/* Something is broken */
+		x11_ok(); /* Reset x11 error status */
 		return FAILURE;
 	}
 
